using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Foo.DataAccessLayer;
using Foo.DataAccessLayer.DataTypes;
using Foo.Platform;
using Foo.GUILib;
using Foo.WorkspaceMgr;

namespace zEasyTest
{
    static class WorkspaceMgr_Test
    {

        static public void TestConfiguration()
        {
            Configurations.Configure_Excel2007();
        }

        static public void TestSingleArtifactLaunch()
        {
            //WorkspaceMgr mgr = new WorkspaceMgr();
            //mgr.LaunchArtifact(DataTypeHelpers.CreateLocationOnlyArtifact(@"D:\Users\HPinvent\Documents\KeepFocusedTask.txt"));
            //mgr.LaunchArtifact(DataTypeHelpers.CreateLocationOnlyArtifact(@"http://www.google.com/"));
        }

        static public void TestWorkspaceLaunch()
        {
            WorkspaceMgr mgr = new WorkspaceMgr();
            
            string strTestWorkspace = "Presentations";

            mgr.LaunchWorkspace(strTestWorkspace);

            System.Threading.Thread.Sleep(30000);

            mgr.CloseWorkspace(strTestWorkspace);


            //mgr.LaunchArtifact(DataTypeHelpers.CreateLocationOnlyArtifact(@"D:\Users\HPinvent\Documents\KeepFocusedTask.txt"));
        }

    }
}
